# iDQ

This repository houses the iDQ source code including documentation and examples.
Please contact Reed Essick (reed.essick@ligo.org) with questions.

Instructions for installing and running iDQ:

  * [Installation](https://docs.ligo.org/reed.essick/iDQ/installation.html)
  * [Running Batch Jobs](https://docs.ligo.org/reed.essick/iDQ/tutorials/running_batch_pipeline.html)
  * [Running Streaming Jobs](https://docs.ligo.org/reed.essick/iDQ/tutorials/running_stream_pipeline.html)

Full documentation can be found here: https://docs.ligo.org/reed.essick/iDQ/

## Design Document

Currently a work-in-progress, but is available here

  - https://www.overleaf.com/10661300shbwgjyyqgvz#/39942261/

We will attempt to keep an up-to-date copy within this repository's documentation and on the DCC.

## Formal documentation

Formal documentation will be managed via Sphinx and should be automatically incorporated into the git repo (e.g.: ReadTheDocs).

## Dependencies

Besides the standard built-in Python libraries (based on Python2.7)

### Functionality/Operation

LDG-standard python packages:
  - numpy
  - h5py
    - used to read/write data to disk

Non-standard python packages:
  - scikit-learn ([v0.18.2](https://pypi.python.org/pypi/scikit-learn/0.18.2))
    - used within some MLA classifiers
  - cython ([v0.25.2](https://pypi.python.org/pypi/Cython/0.25.2))
    - dependency for scikit-learn
  - confluent-kafka-python ([v0.11.5](https://github.com/confluentinc/confluent-kafka-python/releases/tag/v0.11.5))
    - Python bindings for Kafka

Non-python packages:
  - Kafka ([v1.1.1, Scala 2.11](http://kafka.apache.org/))
    - dependency for streaming processes
  - librdkafka([v0.11.5](https://github.com/edenhill/librdkafka/releases/tag/v0.11.5))
    - Kafka C/C++ client library

### Documentation 

  - sphinx

### Testing

  - pytest ([v.2.7.0+](https://docs.pytest.org/))

-------------------------------------------------

## For Developers

### Python Style Guide

  - Objects should return themselves when there is no other obvious data product. For example, 
    - SupervisedClassifier.train should build an internal model and then return a copy of itself whereas 
    - SupervisedClassifier.timeseries should return a list of arrays of ranks, one element for each segment requested (contained within the classifier_data object passed)
  - spaces instead of tabs, with 4-spaces per indent block
  - if there are more than 3 input arguments for a function call, the call should be broken across multiple lines with appropriate indentation
  - all functions and method names should follow the snake case convention, e.g. foo_bar(), where underscores are used to separate names

### Data formats

As much as possible, data written to disk should follow the Common Data Format recommendations
    - https://wiki.ligo.org/DetChar/CommonDataFormat
However, within memory we will not require any particular data format. For example, we will allow ourselves to instantiate and pass around numpy structured arrays instead of hd5f dataset objects even though we will always represent the numpy arrays as datasets when writing them to disk.

### Current stated interests

The following are rough associations between developers and key implementation tasks

  - Reed
    - classifier objects
    - incremental/streaming training
    - calibration

  - Patrick
    - data discovery
      - bottle routes and data transfer
    - data server
    - incremental/streaming training

  - Sydney
    - data discovery
    - anything else that needs help...

### Immediate Action Items

  - these are managed via Issues and Milestones within the git.ligo.org repository

### Deadlines

These will also be managed by the Issues and Milestones within the git.ligo.org repository. However, this list should provide a brief snap-shot of the long-term development schedule.

  - mid October 2017
    - choices of all major technologies made
      - a single specific solution for (S)MDS should be agreed upon
    - assignment of key development tasks to volunteers
      - definitions of Unit Tests and "metrics of success" defined for all key tasks

  - gstlal F2F at PSU : February 2018
    - all major pieces of code in place if not fully debugged/reviewed
      - ETG/feature extraction
      - classifier objects
      - idq-realtime executable
      - idq-train executable
      - idq-calibration executable
      - (S)MDS and appropriate connections between all asynchronous jobs
    - review team identified

  - Sonoma State LVC meeting : March 2018
    - working prototype deployed at each site and streaming predictions to CIT
    - review started
